mode(1)
//
// Demo of fmincon.sci
//

// A basic case :
// we provide only the objective function and the nonlinear constraint
// function : we let fmincon compute the gradients by numerical
// derivatives.
function f = objfun ( x )
f = exp(x(1))*(4*x(1)^2 + 2*x(2)^2 + 4*x(1)*x(2) + 2*x(2) + 1)
endfunction
function [c, ceq] = confun(x)
// Nonlinear inequality constraints
c = [
1.5 + x(1)*x(2) - x(1) - x(2)
-x(1)*x(2) - 10
]
// Nonlinear equality constraints
ceq = []
endfunction
// The initial guess
x0 = [-1,1];
// The expected solution : only 4 digits are guaranteed
xopt = [-9.547345885974547   1.047408305349257]
fopt = 0.023551460139148
// Run fmincon
[x,fval,exitflag,output,lambda,grad,hessian] = fmincon ( objfun,x0,[],[],[],[],[],[], confun )
halt()   // Press return to continue
 
// A case where we provide the gradient of the objective
// function and the Jacobian matrix of the constraints.
// The objective function and its gradient
function [f,G] = objfungrad ( x )
[lhs,rhs]=argn()
f = exp(x(1))*(4*x(1)^2+2*x(2)^2+4*x(1)*x(2)+2*x(2)+1)
if ( lhs  > 1 ) then
G = [
f + exp(x(1)) * (8*x(1) + 4*x(2))
exp(x(1))*(4*x(1)+4*x(2)+2)
]
end
endfunction
// The nonlinear constraints and the Jacobian matrix of the constraints
function [c,ceq,DC,DCeq] = confungrad(x)
// Inequality constraints
c(1) = 1.5 + x(1) * x(2) - x(1) - x(2)
c(2) = -x(1) * x(2)-10
// No nonlinear equality constraints
ceq=[]
[lhs,rhs]=argn()
if ( lhs > 2 ) then
// DC(:,i) = gradient of the i-th constraint
// DC = [
//   Dc1/Dx1  Dc2/Dx1
//   Dc1/Dx2  Dc2/Dx2
//   ]
DC= [
x(2)-1, -x(2)
x(1)-1, -x(1)
]
DCeq = []
end
endfunction
// Test with both gradient of objective and gradient of constraints
options = optimset("GradObj","on","GradConstr","on");
// The initial guess
x0 = [-1,1];
// The expected solution : only 4 digits are guaranteed
xopt = [-9.547345885974547   1.047408305349257]
fopt = 0.023551460139148
// Run fmincon
[x,fval,exitflag,output] = fmincon(objfungrad,x0,[],[],[],[],[],[], confungrad,options)
halt()   // Press return to continue
 
// A case where we set the bounds of the optimization.
// By default, the bounds are set to infinity.
function f = objfun ( x )
f = exp(x(1))*(4*x(1)^2 + 2*x(2)^2 + 4*x(1)*x(2) + 2*x(2) + 1)
endfunction
function [c, ceq] = confun(x)
// Nonlinear inequality constraints
c = [
1.5 + x(1)*x(2) - x(1) - x(2)
-x(1)*x(2) - 10
]
// Nonlinear equality constraints
ceq = []
endfunction
// The initial guess
x0 = [-1,1];
// The expected solution
xopt = [0   1.5]
fopt = 8.5
// Make sure that x(1)>=0, and x(2)>=0
lb = [0,0];
ub = [ ];
// Run fmincon
[x,fval] = fmincon ( objfun , x0,[],[],[],[],lb,ub,confun)
halt()   // Press return to continue
 
//========= E N D === O F === D E M O =========//
//
// Load this script into the editor
//
filename = "fmincon.sce";
dname = get_absolute_file_path(filename);
editor ( dname + filename );
